import * as Comlink from "comlink";
import { AbstractModelDB, parseConfig, } from "@canvas-js/modeldb";
import assert from "assert";
export class OpfsModelDB extends AbstractModelDB {
    static async initialize({ path, models }) {
        const config = parseConfig(models);
        const worker = new Worker("./worker.js", { type: "module" });
        const initializeDB = Comlink.wrap(worker);
        const wrappedDB = (await initializeDB(path, config));
        return new OpfsModelDB({ worker, wrappedDB, config });
    }
    constructor({ worker, wrappedDB, config, }) {
        super(config);
        this.subscriptionId = 0;
        this.worker = worker;
        this.wrappedDB = wrappedDB;
    }
    async close() {
        this.log("closing");
        this.worker.terminate();
    }
    async apply(effects) {
        return this.wrappedDB.apply(effects);
    }
    async get(modelName, key) {
        return this.wrappedDB.get(modelName, key);
    }
    async *iterate(modelName) {
        return this.wrappedDB.iterate(modelName);
    }
    async count(modelName) {
        return this.wrappedDB.count(modelName);
    }
    async query(modelName, query = {}) {
        return this.wrappedDB.query(modelName, query);
    }
    subscribe(modelName, query, callback) {
        const model = this.models[modelName];
        assert(model !== undefined, `model ${modelName} not found`);
        const filter = this.getEffectFilter(model, query);
        const id = this.subscriptionId++;
        // this is async but don't wait for it
        this.wrappedDB.subscribe(id, modelName, query, Comlink.proxy(filter), Comlink.proxy(callback));
        return {
            id,
            results: this.query(modelName, query).then((results) => Promise.resolve(callback(results)).then(() => results, (err) => {
                this.log.error(err);
                return results;
            })),
        };
    }
}
