import sqlite3InitModule from "@sqlite.org/sqlite-wasm";
import { AbstractModelDB, parseConfig, } from "@canvas-js/modeldb";
import { InnerModelDB } from "./InnerModelDB.js";
import assert from "assert";
export class TransientModelDB extends AbstractModelDB {
    static async initialize({ models }) {
        const config = parseConfig(models);
        const sqlite3 = await sqlite3InitModule({ print: console.log, printErr: console.error });
        const db = new sqlite3.oo1.DB();
        const wrappedDB = new InnerModelDB(db, config);
        return new TransientModelDB({ wrappedDB, config });
    }
    constructor({ wrappedDB, config }) {
        super(config);
        this.subscriptionId = 0;
        this.wrappedDB = wrappedDB;
    }
    async close() {
        this.log("closing");
        this.wrappedDB.close();
    }
    async apply(effects) {
        return this.wrappedDB.apply(effects);
    }
    async get(modelName, key) {
        return Promise.resolve(this.wrappedDB.get(modelName, key));
    }
    async *iterate(modelName) {
        return this.wrappedDB.iterate(modelName);
    }
    async count(modelName) {
        return this.wrappedDB.count(modelName);
    }
    async query(modelName, query = {}) {
        return Promise.resolve(this.wrappedDB.query(modelName, query));
    }
    subscribe(modelName, query, callback) {
        const model = this.models[modelName];
        assert(model !== undefined, `model ${modelName} not found`);
        const filter = this.getEffectFilter(model, query);
        const id = this.subscriptionId++;
        // this is async but don't wait for it
        this.wrappedDB.subscribe(id, modelName, query, filter, callback);
        return {
            id,
            results: this.query(modelName, query).then((results) => Promise.resolve(callback(results)).then(() => results, (err) => {
                this.log.error(err);
                return results;
            })),
        };
    }
}
